########################################################################
#  D.Duhaut 5/11/2019
#  Programme de detection d'un robot et d'une balle
#  Et envoie de consigne au robot sur ESP8266
# ###################################################################### 



import numpy as np
import cv2
import cv2.aruco as aruco
import matplotlib.pyplot as plt
import sys
import asyncio
import websockets
import time
import json
import time
from math import *
import math
# Extraction des coefficients de la droite avec 2 points
def calculDroite (x,y,xp,yp):
    a= (y-yp)/(x-xp)
    b= yp-a*xp
    return a,b
# Calcul le position derrire la balle dans laquelle le robot doit taper
def pointDestination(a,b,x):
    eps = 40        #le but est a gauche sinon c'est -40
    return x+eps, a*(x+eps)+b
# Calcul de l'angle entre deux droites pour savoir de combien le robot doit tourner
def angle (a,b,ap,bp):
    return math.atan ((ap-a)/(1+(ap*a)))

def min(a,b,c,d):
    if a<b and a<c and a<d:
        return a
    if b<a and b<c and b<d:
        return b
    if c<a and c<b and c<d:
        return c
    if d<a and d<b and d<c:
        return d

def max(a,b,c,d):
    if a>b and a>c and a>d:
        return a
    if b>a and b>c and b>d:
        return b
    if c>a and c>b and c>d:
        return c
    if d>a and d>b and d>c:
        return d
# Distance euclidienne entre deux points
def distance (a1,b1,a2,b2):
    return sqrt ((a1-a2)*(a1-a2)+(b1-b2)*(b1-b2))

# Calcul du centre des 4 points des 4 coins du carré vu par la caméra
def centre(x,y,a,b,c,d,e,f):
    return min(x,a,c,e)+((max(x,a,c,e)-min(x,a,c,e))/2),(min(y,b,d,f)+max(y,b,d,f))/2

# Extraction centre des markers du robot (3 et 4 a changer en fonction du marker)
def positionRobot(t,ids):
    b=0
    while b<(len(ids)):
        if ids[b][0]==3:
            x0=t[b][0][0][0]
            y0=t[b][0][0][1]
            x1=t[b][0][1][0]
            y1=t[b][0][1][1]
            x2=t[b][0][2][0]
            y2=t[b][0][2][1]
            x3=t[b][0][3][0]
            y3=t[b][0][3][1]
            x3,y3 =centre (x0,y0,x1,y1,x2,y2,x3,y3)
        if ids[b][0]==4 :
            x0=t[b][0][0][0]
            y0=t[b][0][0][1]
            x1=t[b][0][1][0]
            y1=t[b][0][1][1]
            x2=t[b][0][2][0]
            y2=t[b][0][2][1]
            x5=t[b][0][3][0]
            y5=t[b][0][3][1]
            x4,y4 =centre (x0,y0,x1,y1,x2,y2,x5,y5)
        b=b+1
    return x3,y3,x4,y4

# Extraction centre des markers de la balle (9 a changer en fonction du marker)
def positionBalle(t,ids):
    b=0
    while b<(len(ids)):
        if ids[b][0]==9:
            x0=t[b][0][0][0]
            y0=t[b][0][0][1]
            x1=t[b][0][1][0]
            y1=t[b][0][1][1]
            x2=t[b][0][2][0]
            y2=t[b][0][2][1]
            x3=t[b][0][3][0]
            y3=t[b][0][3][1]
            x3,y3 =centre (x0,y0,x1,y1,x2,y2,x3,y3)
        b=b+1
    return x3,y3
###############################################################################
#    Programme principal
###############################################################################
async def main():
    cap = cv2.VideoCapture(1)   # hanger le numero en fonction de la camera a utiliser

    async with websockets.connect('ws://192.168.4.1:4500') as websocket: 
        a=0
        print('go')
        while a<1:

            while(True):
            
                ret, frame = cap.read()
                gray = cv2.cvtColor(frame, cv2.COLOR_BGR2GRAY)

                aruco_dict = aruco.Dictionary_get(aruco.DICT_6X6_250)
                parameters =  aruco.DetectorParameters_create()

                corners, ids, rejectedImgPoints = aruco.detectMarkers(gray, aruco_dict, parameters=parameters)
                # corners donne la liste des points des carrés lu par aruco
                # ids donne les numeros de markers lu
                
                frame_markers = aruco.drawDetectedMarkers(frame.copy(), corners, ids)
                gray = aruco.drawDetectedMarkers(gray, corners)
                # affichage sur ecran des carrés repéré


                cv2.imshow('frame',gray)
                if cv2.waitKey(1) & 0xFF == ord('q'):
                    break
                    
                #cap.release()
                b=0        
                # print(corners)
                ##### pour voir tous les points
                # print('Nombre de points')
                # print(len(ids))
                #### pour voir tous le numeros détectés
                
                    
                xGoal,yGoal = 100, 300   # position du centre du but a etalonner en fonction de la camera
                xBalle,yBalle = positionBalle(corners,ids) # position de la balle
                
                x,y,xp,yp = positionRobot(corners,ids) # position des deux marqueurs du robot
                xRobot, yRobot = (x+xp)/2, (y+yp)/2 # position du centre du robot

                droiteBalleButA, droiteBalleButB = calculDroite (xBalle,yBalle,xGoal,yGoal)
                # droite qui passe du centre de la balle eu centre du but
                xRobotDestination, yRobotDestination = pointDestination(droiteBalleButA, droiteBalleButB,xBalle)
                # position a atteindre sur la droite balle/but pour etre juste derriere la balle
                droiteCibleA, droiteCibleB = calculDroite(xRobot, yRobot,xRobotDestination, yRobotDestination)
                # droite D qui passe par le robot et sa posisition a atteindre
                droiteRobotA, droiteRobotB = calculDroite(xRobot, yRobot,0, 0)
                # droite qui donne l'orientation du robot (qui permet de calculer l'angle avec la doite D)


                # print(droiteBalleButA, droiteBalleButB)
                # print(xRobotDestination, yRobotDestination)
                # print(droiteRobotA, droiteRobotB)
                # print(angle(droiteRobotA, droiteRobotB,droiteCibleA, droiteCibleB))

                print('Position du robot')
                # print(ids[0][0])
                print(xRobot,yRobot)
                # print(ids[1][0])
                print('Position de la balle')
                print(xBalle,yBalle)
                ##############################################################################
                #  ICI est la partie qui calcule ce que doit faire le robot 
                #  avec toutes les informations sélectionnees  
                ##############################################################################
                # Envoie de la consigne au robot ici 20 sur la roue droite et 20 sur la roue gauche
                print ('pass 1\n')
                x = {
                    "type": "Control",
                    "valR": 20,
                    "valL": 20,
                }
                await websocket.send(json.dumps(x))
                # boucle d'attente
                time.sleep(3)
                print ('pass 2\n')
                # envoie de zero zero sur les deux roues pour arreter le robot
                x = {
                    "type": "Control",
                    "valR": 0,
                    "valL": 0,
                }
                await websocket.send(json.dumps(x))

                # deuxième boucle d'attente et on recommence a saisir un nouvelle position avec la camera
                time. sleep(1.4)
                
            # print(len(ids))

asyncio.get_event_loop().run_until_complete(main())
cv2.destroyAllWindows()