Programme de paramétrage du ESP 8266
#include <ESP8266wifi.h>

/* Application qui permet de paramètrer le module ESP8266 suivant 2 modes:
*
* 1. mode 1= Mode station (module wifi qui a besoin d'un point d'accès type box)
* 2. mode 2= Mode point d'accès autonome
*
* compléter ci dessous les paramètres voulus
*
* Connecter le Shield Grove sur un Arduino Uno
* Connecter le module Wifi grove sur le port D2 (port 2 et 3)
*
* L'application paramètre le module en 9600 bauds automatiquement!
*
* Attention bien sélectionner la bonne vitesse dans le moniteur série (115200!)
* Bien sélectionner l'option en bas à droite: "pas de fin de ligne"
*
* Détails des commandes AT
* http://bbs.espressif.com/download/file.php?id=1085
* Version: 28/2/2016
* Auteur: julien.launay@ac-lyon.fr
*/

#include <SoftwareSerial.h>

SoftwareSerial port_serie_esp8266(2, 3); // RX, TX connecté sur le port D5 du shield pour communiquer entre l'Arduino et l'ESP8266

// à compléter avant de lancer l'application!
//------------------------------------------------------------------------------------

const String adresse_IP= "192.168.0.56"; //adresse IP du module wifi ESP8266
const String adresse_IP_passerelle ="192.168.0.1";
const String masque_IP= "255.255.255.0";
const String mot_passe_point_acces= "92429659";
const String SSID_point_acces= "techno salle5";
const String SSID_esp8266="";
const String mot_passe_esp8266="";

//------------------------------------------------------------------------------------

unsigned long baudrate[] = {1200, 2400, 4800, 9600, 19200, 38400, 57600, 115200};

void setup() {

Serial.begin(115200); // pour communique entre le moniteur série de l'IDE arduino du PC et l'Arduino (port 0 et 1)

}

void loop() {

boolean baudrate_trouve = false;

char mode = '0'; // si mode =1 alors ESP8266 = point accès, si mode=2 alors point d'accès en mode station qui doit se connecter à point d'acces type box

byte index_baudrate=0;

Serial.println("Parametrage automatique du module Wifi ESP8266");
Serial.println("");
Serial.println("Choisissez le mode du module Wifi");
Serial.println("Taper: 1=station 2=point d acces");
Serial.println("");

do {

while (Serial.available()<1) {

//on attent de recevoir le mode sélectionné

}

mode = char(Serial.read());

switch (mode) {

case '1':

Serial.println("Mode station (box necessaire) selectionne");

break;

case '2':

Serial.println("Mode point d acces selectionne");

break;

default:

Serial.println("Mauvais choix: 1=station 2=point d acces");

}

} while((mode!='2') && (mode!='1'));

//recherche de du baudrate automatiquement

Serial.println("Recherche du baudrate du module ESP8266");

while ((baudrate_trouve == false) && (index_baudrate<8 )) {

port_serie_esp8266.begin(baudrate[index_baudrate]);
port_serie_esp8266.println("AT");
Serial.println("Baudrate teste="+String(baudrate[index_baudrate]));

if (reponse()) {

baudrate_trouve=true;
Serial.println("Baudrate trouve="+String(baudrate[index_baudrate]));

//on parametre le module ESP8266 en 9600 bauds

port_serie_esp8266.println("AT+UART_DEF=9600,8,1,0,0");

if (reponse()) {

// paramétrage du module

port_serie_esp8266.begin(9600);
port_serie_esp8266.println("AT+RESTORE");
Serial.println("Remise en configuration usine...");
delay(2000);
port_serie_esp8266.begin(115200);
delay(1000);
port_serie_esp8266.println("AT+UART_DEF=9600,8,1,0,0");
delay(1000);
port_serie_esp8266.begin(9600);
delay(1000);
Serial.println("Module Wifi ESP8266 programme en 9600bds");

//choix du mode de paramétrage mode=1 point d'accès, mode=2 station avec point d'accès distant

switch (mode){

case '1':// mode station qui a besoin d'un point d'accès

port_serie_esp8266.println("AT+CWMODE_DEF=1");

if (reponse()) {

Serial.println("Mode station OK");
String envoi ="AT+CIPSTA_DEF=\"";
envoi+=adresse_IP;
envoi+="\",\"";
envoi+=adresse_IP_passerelle;
envoi+="\",\"";
envoi+=masque_IP;
envoi+="\"";
port_serie_esp8266.println(envoi);

//port_serie_esp8266.println("AT+CIPSTA_DEF=\""+adresse_IP+"\",\""+adresse_IP_passerelle+"\",\""+masque_IP+"\"");

if (reponse()) {

Serial.println("Adresse OK");
port_serie_esp8266.println("AT+CWJAP_DEF=\""+SSID_point_acces+"\",\""+mot_passe_point_acces+"\"");
delay(5000);

//Serial.print(port_serie_esp8266.readString());

if (reponse()) {

Serial.println("SSID OK");
delay(3000);
port_serie_esp8266.println("AT+CWAUTOCONN=1");

if (reponse()) {

Serial.println("Mode autoconnect OK");
Serial.println("Module Wifi Esp8266 parametre correctement mode STATION");
Serial.println();

}

}

}

}

break;

case '2':// mode point d'acces autonome

port_serie_esp8266.println("AT+CWMODE_DEF=2");

if (reponse()) {

delay(1000);
Serial.println("Mode point d acces OK");
String envoi ="AT+CIPAP_DEF=\"";
envoi+=adresse_IP;
envoi+="\",\"";
envoi+=adresse_IP_passerelle;
envoi+="\",\"";
envoi+=masque_IP;
envoi+="\"";
port_serie_esp8266.println(envoi);

if (reponse()) {

Serial.println("Adresse OK");
port_serie_esp8266.println("AT+CWSAP_DEF=\""+SSID_esp8266+"\",\""+mot_passe_esp8266+"\",1,0");

if (reponse()) {

Serial.println("SSID OK");
Serial.println("Module Wifi Esp8266 parametre correctement mode POINT ACCES");
Serial.println();

}

}

}

break;

}

}

}

else {

index_baudrate+=1;

}

}

if (baudrate_trouve == false) {

Serial.println("Impossible de dialoguer avec le module wifi!");

}

}

boolean reponse(){ // attend la réponse OK à une commande AT

if (port_serie_esp8266.find("OK")) {

return true;

}

else {

return false;

}

port_serie_esp8266.flush();
delay(1000);

}

